
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import com.jpackages.jdun.*;

/**
 * Example code using JDUN
 * Copyright (c) 2001-2007
 * Company: VersaEdge Software, LLC
 */

public class Example1 extends JFrame {
  public class DialNotify implements DialingNotification {    

    // callback method
    public void notifyDialingState(String name, int state, int error) {
      // output the meaning of the new dialing state
      System.out.println("notify - " + name + ": (" + state + ") " + DialingState.getDialingStateString(state));

      // if an error, output the meaning of the error code
      if (error != 0) {
        System.out.println("ERROR: " + error + " " + dum.getErrorMessageForCode(error));
      }
    }
  }

  // handle for DialUpManager instance
  DialUpManager dum;
  // instantiate DialNotify class (defined above) which has the callback method
  DialNotify dnot = new DialNotify();

  // GUI definitions
  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JScrollPane jScrollPane1 = new JScrollPane();
  DefaultListModel lm = new DefaultListModel();
  JList jList1 = new JList(lm);
  JPanel jPanel3 = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  JButton jButtonConnect = new JButton();
  JButton jButtonDisconnect = new JButton();
  
  BorderLayout borderLayout3 = new BorderLayout();
  JPanel jPanel4 = new JPanel();
  JLabel jLabel1 = new JLabel();
  JButton jButtonRefresh = new JButton();
  JPanel jPanel5 = new JPanel();
  FlowLayout flowLayout1 = new FlowLayout();
  
  JButton jButtonDelete = new JButton();
  JButton jButtonRename = new JButton();
  JPanel jPanel6 = new JPanel();
  JCheckBox jCheckBox1 = new JCheckBox();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel jPanel9 = new JPanel();
  JPanel jPanel7 = new JPanel();
  JTextField jTextFieldUsername = new JTextField();
  JLabel jLabel2 = new JLabel();
  JPasswordField jPasswordField1 = new JPasswordField();
  JPanel jPanel8 = new JPanel();
  JLabel jLabel3 = new JLabel();
  BorderLayout borderLayout5 = new BorderLayout();
  BorderLayout borderLayout6 = new BorderLayout();
  BorderLayout borderLayout7 = new BorderLayout();
  JPanel jPanel10 = new JPanel();
  JRadioButton jRadioButtonOverride = new JRadioButton();
  JRadioButton jRadioButtonDefault = new JRadioButton();
  JTextField jTextFieldPhoneNumber = new JTextField();
  JLabel jLabel4 = new JLabel();

  //Constructor
  public Example1() {
    try {
      // instantiate the DialUpManager class
      dum = new DialUpManager(dnot);
    }
    catch (LibraryLoadFailedException e) {
      if (e instanceof JDUNLibraryLoadFailedException)
        System.out.println("JDUN Library failed to load");
      else if (e instanceof RASLibraryLoadFailedException)
        System.out.println("RAS Library failed to load");
      System.exit(0);
    }

    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
      initialize();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  // enable username/password override ability
  public void enableParams(boolean b) {
    this.jTextFieldUsername.setEnabled(b);
    this.jTextFieldUsername.setEditable(b);
    this.jPasswordField1.setEnabled(b);
    this.jPasswordField1.setEditable(b);
  }

  // system initializing
  public void initialize() {
    this.enableParams(false);
    System.out.println("initializing...");

    ButtonGroup bg = new ButtonGroup();
    bg.add(this.jRadioButtonDefault);
    bg.add(this.jRadioButtonOverride);

    this.refreshList();
  }

  // populate JList
  public void refreshList() {
    lm.clear();
    try {
      // fetch the entry names
      String[] names = dum.getEntryNames();

      for (int i=0; i < names.length; i++) {
        lm.addElement(names[i]);
      }
      this.jList1.repaint();
    }
    catch (Exception e) {}
  }

  //Component initialization
  private void jbInit() throws Exception  {
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(400, 300));
    this.setTitle("JDUN Example");
    jPanel1.setLayout(borderLayout2);
    jScrollPane1.setPreferredSize(new Dimension(260, 80));
    jPanel1.setPreferredSize(new Dimension(260, 100));
    jButtonConnect.setPreferredSize(new Dimension(105, 24));
    jButtonConnect.setText("Connect");
    jButtonConnect.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonConnect_actionPerformed(e);
      }
    });
    jButtonDisconnect.setPreferredSize(new Dimension(105, 24));
    jButtonDisconnect.setText("Disconnect");
    jButtonDisconnect.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonDisconnect_actionPerformed(e);
      }
    });    
    jPanel3.setLayout(borderLayout3);
    jLabel1.setText("Dial-up Entry Names");
    jButtonRefresh.setPreferredSize(new Dimension(79, 24));
    jButtonRefresh.setText("Refresh");
    jButtonRefresh.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonRefresh_actionPerformed(e);
      }
    });
    jPanel5.setPreferredSize(new Dimension(70, 28));
    jPanel5.setLayout(flowLayout1);
    flowLayout1.setVgap(2);    
    jButtonDelete.setPreferredSize(new Dimension(60, 24));
    jButtonDelete.setMargin(new Insets(0, 0, 0, 0));
    jButtonDelete.setText("Delete");
    jButtonDelete.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonDelete_actionPerformed(e);
      }
    });
    jButtonRename.setPreferredSize(new Dimension(70, 24));
    jButtonRename.setMargin(new Insets(0, 0, 0, 0));
    jButtonRename.setText("Rename");
    jButtonRename.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jButtonRename_actionPerformed(e);
      }
    });
    jPanel6.setBorder(BorderFactory.createEtchedBorder());
    jPanel6.setLayout(borderLayout4);
    jCheckBox1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jCheckBox1_actionPerformed(e);
      }
    });
    jList1.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        jList1_valueChanged(e);
      }
    });
    jTextFieldUsername.setPreferredSize(new Dimension(200, 24));
    jLabel2.setPreferredSize(new Dimension(65, 17));
    jLabel2.setText("Username");
    jPasswordField1.setPreferredSize(new Dimension(200, 24));
    jLabel3.setPreferredSize(new Dimension(65, 17));
    jLabel3.setText("Password");
    jPanel9.setLayout(borderLayout5);
    jPanel8.setLayout(borderLayout7);
    jPanel7.setLayout(borderLayout6);
    jRadioButtonOverride.setText("Override");
    jRadioButtonOverride.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jRadioButtonOverride_actionPerformed(e);
      }
    });
    jRadioButtonDefault.setSelected(true);
    jRadioButtonDefault.setText("Default");
    jRadioButtonDefault.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        jRadioButtonDefault_actionPerformed(e);
      }
    });
    jTextFieldPhoneNumber.setPreferredSize(new Dimension(120, 24));
    jTextFieldPhoneNumber.setEditable(false);
    jLabel4.setText("Phone Number");
    contentPane.add(jPanel1,  BorderLayout.CENTER);
    jPanel1.add(jScrollPane1, BorderLayout.CENTER);
    jPanel1.add(jPanel3, BorderLayout.NORTH);
    jPanel3.add(jPanel4, BorderLayout.CENTER);
    jPanel4.add(jLabel1, null);
    jPanel4.add(jButtonRefresh, null);
    jPanel3.add(jPanel5,  BorderLayout.SOUTH);    
    jPanel5.add(jButtonDelete, null);
    jPanel5.add(jButtonRename, null);
    jPanel1.add(jPanel6,  BorderLayout.SOUTH);
    jPanel6.add(jCheckBox1, BorderLayout.WEST);
    jPanel6.add(jPanel9,  BorderLayout.CENTER);
    jPanel8.add(jLabel3, BorderLayout.WEST);
    jPanel8.add(jPasswordField1, BorderLayout.CENTER);
    jPanel9.add(jPanel8,  BorderLayout.SOUTH);
    jPanel9.add(jPanel7,  BorderLayout.NORTH);
    jPanel7.add(jLabel2, BorderLayout.WEST);
    jPanel7.add(jTextFieldUsername, BorderLayout.CENTER);
    jPanel6.add(jPanel10,  BorderLayout.SOUTH);
    jPanel10.add(jLabel4, null);
    jPanel10.add(jRadioButtonDefault, null);
    jPanel10.add(jRadioButtonOverride, null);
    jPanel10.add(jTextFieldPhoneNumber, null);
    jScrollPane1.getViewport().add(jList1, null);
    contentPane.add(jPanel2,  BorderLayout.SOUTH);
    jPanel2.add(jButtonConnect, null);
    jPanel2.add(jButtonDisconnect, null);
  }
  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }

  // refresh the JList
  void jButtonRefresh_actionPerformed(ActionEvent e) {
    this.refreshList();
  }

  // connect the selected entry
  void jButtonConnect_actionPerformed(ActionEvent e) {
    String entryName = (String) this.jList1.getSelectedValue();
    if (entryName == null)
      return;

    // fetch override phone number
    String phonenum = null;
    if (this.jRadioButtonOverride.isSelected()) {
      phonenum = this.jTextFieldPhoneNumber.getText();
    }


    if (!jCheckBox1.isSelected()) {
      // dial the entry name with its default username/password
      if (phonenum == null) {
        dum.dialEntryAsynchronous(entryName);
      }
      else {
        dum.dialEntryAsynchronous(entryName, phonenum);
      }
    }
    else {
      // get the overriding username/password and use those for dialing
      String username = this.jTextFieldUsername.getText();
      String password = new String(this.jPasswordField1.getPassword());
      if (phonenum == null) {
        dum.dialEntryAsynchronous(entryName, username, password);
      }
      else {
        dum.dialEntryAsynchronous(entryName, username, password, "", phonenum);
      }
    }
  }

  // hangup the selected entry
  void jButtonDisconnect_actionPerformed(ActionEvent e) {
    final String entryName = (String) this.jList1.getSelectedValue();
    if (entryName == null)
      return;

    dum.hangUpEntry(entryName);
  }

  // delete the selected entry
  void jButtonDelete_actionPerformed(ActionEvent e) {
    String entryName = (String) this.jList1.getSelectedValue();
    if (entryName == null)
      return;

    // confirm the deletion
    int reply = JOptionPane.showConfirmDialog(this,
                            "Are you sure you want to delete " + entryName + "?",
                            "Confirm Delete" ,
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.PLAIN_MESSAGE);
    if (reply == JOptionPane.NO_OPTION) {
      return;
    }

    // confirmed, so delete the entry
    dum.deleteEntry(entryName);
    // update JList after the selected entry is deleted
    this.refreshList();
  }

  // rename the selected entry
  void jButtonRename_actionPerformed(ActionEvent e) {
    String entryName = (String) this.jList1.getSelectedValue();
    if (entryName == null)
      return;

    // obtain the new name
    String message = "Enter new entry name for '" + entryName + "'";
    String newname = (String) JOptionPane.showInputDialog(this, message, "Rename Entry", JOptionPane.PLAIN_MESSAGE, null, null, entryName);
    if (newname == null)
      return;
    if (newname.equals(entryName))
      return;

    // rename from entryName to newname
    dum.renameEntry(entryName, newname);
    // update JList after the selected entry is renamed
    this.refreshList();

  }

  void jCheckBox1_actionPerformed(ActionEvent e) {
    if (jCheckBox1.isSelected())
      this.enableParams(true);
    else
      this.enableParams(false);
  }

  // when a list selection is made, load the appropriate username/password
  void jList1_valueChanged(ListSelectionEvent e) {
    String entryName = (String) this.jList1.getSelectedValue();
    if (entryName == null)
      return;

    // fetch the username/password
    String password = dum.getPassword(entryName);
    String username = dum.getUsername(entryName);

    // display the username/password
    this.jTextFieldUsername.setText(username);
    this.jPasswordField1.setText(password);

    // fetch the entry properties
    DialUpEntryProperties props = dum.getDialUpEntryProperties(entryName);

    // display the phone number
    if (props.getUseCountryAndAreaCodes()) {
      String areacode = props.getAreaCode();
      String phonenum = props.getLocalPhoneNumber();
      this.jTextFieldPhoneNumber.setText(areacode + phonenum);
    }
    else {
      this.jTextFieldPhoneNumber.setText(props.getLocalPhoneNumber());
    }
  }

  void phoneButtonChange() {
    if (this.jRadioButtonDefault.isSelected()) {
      this.jTextFieldPhoneNumber.setEditable(false);
    }
    else {
      this.jTextFieldPhoneNumber.setEditable(true);
    }
  }

  void jRadioButtonDefault_actionPerformed(ActionEvent e) {
    phoneButtonChange();
  }

  void jRadioButtonOverride_actionPerformed(ActionEvent e) {
    phoneButtonChange();
  }

  //Main method
  public static void main(String[] args) {
    Example1 frame = new Example1();

    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame.getSize();
    if (frameSize.height > screenSize.height) {
      frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
      frameSize.width = screenSize.width;
    }
    frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    frame.setVisible(true);
  }
}
